/*
 * Copyright (c) 1992 Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms are permitted
 * provided that the above copyright notice and this paragraph are
 * duplicated in all such forms and that any documentation,
 * advertising materials, and other materials related to such
 * distribution and use acknowledge that the software was developed
 * by the University of California, Lawrence Berkeley Laboratory,
 * Berkeley, CA.  The name of the University may not be used to
 * endorse or promote products derived from this software without
 * specific prior written permission.
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 */

#ifndef lint
static char rcsid[] =
    "@(#) $Header: bpf.c,v 1.3 92/06/08 02:55:42 van Exp $ (LBL)";
#endif

#include <stdio.h>
#include <ctype.h>

#include <sys/types.h>
#include <sys/file.h>
#include <sys/time.h>
#include <errno.h>
#include <sys/ioctl.h>
#include <net/bpf.h>
#include <sys/socket.h>
#include <net/if.h>

static char *tcpdumps[] = {
	"tcpdump",
	"/usr/etc/tcpdump",
	"/usr/sbin/tcpdump",
	"/sbin/tcpdump",
	0
};

#define BPF_MAXLEN 256

static int tcpdump_child;

static void
closepipe(fp)
	FILE *fp;
{
	int pid;

	fclose(fp);
	while (1) {
		pid = wait((int *)0);
		if (pid == tcpdump_child)
			return;
		if (pid == -1) {
			if (errno == EINTR)
				continue;
			fprintf(stderr, "problem with tcpdump child\n");
			exit(1);
		}
	}
}

static FILE *
openpipe(ifname, snaplen, expr)
	char *ifname;
	int snaplen;
	char *expr;
{
	char **cp;
	FILE *fp;
	int pid;
	int p[2];
	char num[12];
	char *argv[8];

	if (pipe(p) < 0)
		return 0;

	pid = fork();
	if (pid < 0)
		return 0;

	if (pid == 0) {
		argv[0] = "tcpdump";
		argv[1] = "-ddd";
		argv[2] = "-i";
		argv[3] = ifname;
		argv[4] = "-s";
		argv[5] = num;
		argv[6] = expr;
		argv[7] = 0;
		
		close(p[0]);
		dup2(p[1], 1);
		if (p[1] != 1)
			close(p[1]);

		sprintf(num, "%d", snaplen);
		for (cp = tcpdumps; *cp != 0; ++cp)
			execvp(*cp, argv);
		_exit(0);
	}
	tcpdump_child = pid;
	close(p[1]);
	fp = fdopen(p[0], "r");
	if (fp == 0)
		close(p[0]);

	return fp;
}

static 
read_filter(ifname, expr, prog, snaplen)
	char *ifname;
	char *expr;
	struct bpf_program *prog;
	int snaplen;
{
	int jt, jf, len, pid;
	FILE *fp;
	struct bpf_insn *insn;

	fp = openpipe(ifname, snaplen, expr);
	if (fp == 0)
		return -1;
	if (fscanf(fp, "%d", &len) != 1) {
		closepipe(fp);
		return -1;
	}
	prog->bf_len = len;
	if (len >= BPF_MAXLEN)
		return 0;
	insn = prog->bf_insns;
	while (--len >= 0 &&
	       fscanf(fp, "%hu %u %u %lu", 
		      &insn->code, &jt, &jf, &insn->k) == 4) {
		insn->jt = jt;
		insn->jf = jf;
		++insn;
	}
	(void)closepipe(fp);
	return 0;
}

#define MAX_BPFLEN 256

int
bpf_init(ifname, filter, bufsize, snaplen)
	char *ifname;
	char *filter;
	long *bufsize;
	int snaplen;
{
	int fd;
	int n = 0;
	struct ifreq ifr;
	struct bpf_program bpf;
	struct bpf_insn insns[MAX_BPFLEN];
	char dev[64];
	u_int ui;

	/*
	 * Go through all the minors and find one that isn't in use. 
	 */
	do {
		(void)sprintf(dev, "/dev/bpf%d", n++);
		errno = 0;
		fd = open(dev, O_RDONLY);
	} while (fd < 0 && errno == EBUSY);

	if (fd >= 0) {
		(void)strncpy(ifr.ifr_name, ifname, sizeof(ifr.ifr_name));
		if (ioctl(fd, BIOCSETIF, &ifr) < 0)
			goto err;

		/* Okay if this one fails. */
		(void)ioctl(fd, BIOCPROMISC, (void *)0);

		if (ioctl(fd, BIOCGBLEN, (caddr_t)&ui) < 0)
			goto err;
		*bufsize = ui;

		if (! filter)
			filter = "";
		bpf.bf_insns = insns;
		if (read_filter(ifname, filter, &bpf, snaplen) < 0)
			goto err;

		if (bpf.bf_len == 0)
			goto err;
		if (ioctl(fd, BIOCSETF, (caddr_t)&bpf) < 0)
			goto err;
	}
	return fd;
  err:
	close(fd);
	return (-1);
}
