/*
 * Copyright (c) 1987, 1988, 1989 Stanford University
 *
 * Permission to use, copy, modify, distribute, and sell this software and its
 * documentation for any purpose is hereby granted without fee, provided
 * that the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the name of Stanford not be used in advertising or
 * publicity pertaining to distribution of the software without specific,
 * written prior permission.  Stanford makes no representations about
 * the suitability of this software for any purpose.  It is provided "as is"
 * without express or implied warranty.
 *
 * STANFORD DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
 * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS.
 * IN NO EVENT SHALL STANFORD BE LIABLE FOR ANY SPECIAL, INDIRECT OR
 * CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE,
 * DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
 * OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
 * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

#include <InterViews/sensor.h>
#include <InterViews/world.h>

#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <osfcn.h>

#include "filter.h"
#include "screen.h"
#include "stripchart.h"

/* The way one has to write a stringify macro proves beyond the
 * shadow of a doubt that the Ansi C committee was braindead. */
#define stringify2(s) #s
#define stringify(s) stringify2(s)

static PropertyData props[] = {
	{ "*geometry",		"600x300" },
	{ "*font", 		"*-helvetica-medium-r-normal-*-14-*" },
	{ "*Period", 		"0.2" },
	{ "*FilterGain",	"1.0" },
	{ "*Bandwidth",		"10e6" },
	{ "*DisplayWidth",	"60" },
	{ "*DisplayHeight",	"1" },
	{ "*XGrid",		"0" },
	{ "*YGrid",		"5" },
	{ nil }
};

static OptionDesc options[] = {
    { nil },
};

char *program;

extern "C" int getopt();
extern "C" int optind, opterr;
extern "C" char *optarg;

void
usage()
{
	fprintf(stderr, "usage\n");
	exit(1);
}

extern "C" const char* read_infile(const char* fname);
extern "C" const char* copy_argv(const char* const* argv);
extern "C" const char* lookup_device(void);
extern "C" void error(const char*, ...);

int
main(int argc, char* argv[])
{
	program = argv[0];
	char *cp = rindex(program, '/');
	if (cp != 0)
		program = cp + 1;

	const char* infile = 0;
	World* world = new World(program, props, options, argc, argv);
	const char* device = world->GetAttribute("Interface");
	const char* period = world->GetAttribute("Period");
	const char* gain = world->GetAttribute("FilterGain");
	const char* bandwidth = world->GetAttribute("Bandwidth");
	/* initial dimensions for stripchart. */
	const char* width = world->GetAttribute("DisplayWidth");
	const char* height = world->GetAttribute("DisplayHeight");

	const char* xgrid = world->GetAttribute("XGrid");
	const char* ygrid = world->GetAttribute("YGrid");

	int op;
	optind = 1;
	while ((op = getopt(argc, argv, "b:F:g:h:i:p:w:x:y:")) != -1) {
		switch (op) {

		case 'b':
			bandwidth = optarg;
			break;

		case 'F':
			infile = optarg;
			break;

		case 'h':
			height = optarg;
			break;

		case 'g':
			gain = optarg;
			break;

		case 'i':
			device = optarg;
			break;

		case 'p':
			period = optarg;
			break;

		case 'w':
			width = optarg;
			break;

		case 'x':
			xgrid = optarg;
			break;

		case 'y':
			ygrid = optarg;
			break;

		default:
			usage();
			break;
		}
	}
	const char* cmdbuf;
	if (infile != 0) 
		cmdbuf = read_infile(infile);
	else
		cmdbuf = copy_argv(&argv[optind]);

	if (device == 0) {
		device = lookup_device();
		if (device == 0)
			error("can't find any interfaces");
	}
	PacketFilter* filter = new PacketFilter(device, cmdbuf);

	StripChart* chart = new StripChart(cmdbuf != 0? cmdbuf : "All Traffic",
					   atof(width), atof(height),
					   atoi(xgrid), atoi(ygrid));
	Screen* screen = new Screen(filter, chart, atof(period),
				    atof(gain), atof(bandwidth)/8.);

	world->InsertApplication(screen);
	screen->Run();
	delete filter;
	delete chart;
	delete screen;
	delete world;
	
	return (0);
}
